#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os.path
import dsmz


class PNU(dsmz.API):
    def __init__(self, user, password, url_prefix='https://bacdive.dsmz.de/api/pnu'):
        dsmz.API.__init__(self, user, password, url_prefix)

    def all_species(self, fmt=dsmz.Format.JSON):
        return self.request_service('/species/', fmt)

    def all_genera(self, fmt=dsmz.Format.JSON):
        return self.request_service('/genus/', fmt)

    def all_families(self, fmt=dsmz.Format.JSON):
        return self.request_service('/fam/', fmt)

    def all_classes(self, fmt=dsmz.Format.JSON):
        return self.request_service('/class/', fmt)

    def all_reclassified(self, fmt=dsmz.Format.JSON):
        return self.request_service('/reclassified/', fmt)

    def taxon(self, taxon_name, fmt=dsmz.Format.JSON):
        return self.request_service('/taxon/%s/' % taxon_name, fmt)

    @staticmethod
    def _build_name_service(genus, species=None, subspecies=None):
        if species is None:
            if subspecies is not None:
                raise RuntimeError('subspecies epithet specified but no species epithet')
            return '/name/%s/' % genus
        if subspecies is None:
            return '/name/%s/%s/' % (genus, species)
        return '/name/%s/%s/subsp/%s/' % (genus, species, subspecies)

    def name(self, genus, species=None, subspecies=None, fmt=dsmz.Format.JSON):
        return self.request_service(PNU._build_name_service(genus, species, subspecies), fmt)

    def culturecollectionno(self, n, fmt=dsmz.Format.JSON):
        return self.request_service('/culturecollectionno/%s/' % str(n), fmt)


class PNUCLI(dsmz.CLI):
    def __init__(self):
        dsmz.CLI.__init__(self, description='CLI client for PNU services')
        self.query_group = self.add_mutually_exclusive_group()
        self.query_group.add_argument('-l', '--all', action='store_const', const='all', dest='query', default='all', help='query all species|genera|families|classes|reclassified <>')

    def default_url_prefix(self):
        return 'https://bacdive.dsmz.de/api/pnu'

    def run(self):
        args = self.get_args()
        pnu = PNU(args.user, args.password, url_prefix=args.url_prefix)
        if args.query == 'all':
            PNUCLI.listall(pnu, args.fmt, args.outdir, args.terms)

    @staticmethod
    def listall(pnu, fmt, outdir, terms):
        rank = terms.pop(0)
        response = dsmz.CLI._response_or_log(getattr(pnu, 'all_' + rank)(dsmz.Format.JSON))
        n = 1
        while response is not None:
            dsmz.CLI._save_response_content(response, os.path.join(outdir, '%s-%02d%s' % (rank, n, dsmz.Format.JSON.ext)))
            j = response.json()
            next_page_url = j['next']
            if next_page_url is None:
                break
            response = dsmz.CLI._response_or_log(pnu.request_raw_url(next_page_url))
            n += 1


if __name__ == '__main__':
    PNUCLI().run()
